# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: octave_sdk/grpc/quantummachines/octave/api/v1/api.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterable,
    AsyncIterator,
    Dict,
    Iterable,
    List,
    Optional,
    Union,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class OctaveModule(betterproto.Enum):
    OCTAVE_MODULE_UNSPECIFIED = 0
    OCTAVE_MODULE_RF_UPCONVERTER = 1
    OCTAVE_MODULE_RF_DOWNCONVERTER = 2
    OCTAVE_MODULE_IF_DOWNCONVERTER = 3
    OCTAVE_MODULE_SYNTHESIZER = 4
    OCTAVE_MODULE_MOTHERBOARD = 5
    OCTAVE_MODULE_SOM = 6


class ConstantSource(betterproto.Enum):
    CONSTANT_SOURCE_UNSPECIFIED = 0
    CONSTANT_SOURCE_50_OHM = 1
    CONSTANT_SOURCE_SHORT = 2
    CONSTANT_SOURCE_OPEN = 3


class AdminInfoRequestOpxType(betterproto.Enum):
    OPX_TYPE_UNSPECIFIED = 0
    OPX_TYPE_OPX_PLUS = 1
    OPX_TYPE_OPX_1000 = 2
    OPX_TYPE_NONE = 3


class MonitorResponseOctaveError(betterproto.Enum):
    OCTAVE_ERROR_UNSPECIFIED = 0
    OCTAVE_ERROR_BAD_EXTERNAL_CLOCK = 1
    OCTAVE_ERROR_BAD_INTERNAL_CLOCK = 2
    OCTAVE_ERROR_FPGA_NOT_FOUND = 3
    OCTAVE_ERROR_FPGA_HW_FAIL = 4
    OCTAVE_ERROR_FPGA_BITFILE_FAIL = 5
    OCTAVE_ERROR_MODULE_NOT_FOUND = 6
    OCTAVE_ERROR_OVERHEAT = 7
    OCTAVE_ERROR_OVERHEAT_PROTECTION = 8
    OCTAVE_ERROR_READ_TEMP_FAIL = 9
    OCTAVE_ERROR_PLL_NOT_LOCKED = 10


class ControlResponseRdataDebug(betterproto.Enum):
    RDATA_DEBUG_UNSPECIFIED = 0
    RDATA_DEBUG_ERROR_RESPONSE = 160
    RDATA_DEBUG_SUCCESS_RESPONSE = 32


class RfUpConvUpdateIfInputSelection(betterproto.Enum):
    """
    IF input   One can select two different attenuations for each input channel
    independently (which does not make too much sense). In the current setup,
    ATTN_1 to just a bypass and ATTN_2 is 10dB.           | ATTN_1 | ATTN_2
    |  (0dB) | (10dB)   --------+--------+--------   SW1 (Q) | HIGH   | LOW
    SW2 (I) | HIGH   | LOW
    """

    IF_INPUT_SELECTION_UNSPECIFIED = 0
    IF_INPUT_SELECTION_PASS_THROUGH = 1
    IF_INPUT_SELECTION_ATTENUATE_10DB = 2


class RfUpConvUpdateFastSwitchMode(betterproto.Enum):
    """
    RF output switch   The output of the up conversion circuit goes through a
    fast switch. It    can be controlled in various ways. Some of them depends
    on the board assembly.   Control sources:   1. GPIO        Will allow
    either a slow controlled state, or HIGH_Z to allow override.          | ON
    | OFF  | OVERRIDE      ----+------+------+----------        SW4 | LOW  |
    HIGH | HIGH_Z      This requires the appropriate resistor should be
    mounted. If not, then      it is the same as HIGH_Z   2. DIRECT        This
    is a signal that comes from an external SMA. If the external SMA goes
    directly        to the switch, it will have a pull-down resistor. In that
    case, if it is not connected,        the switch will be ON. If the external
    SMA is routed to the FPGA, the FPGA will pass it        as is to the
    switch.     3. INVERTED         This is relevant only if the external SMA
    is routed to the FPGA. In that case, the         FPGA will pass it to the
    switch after inversion.   We will either use the FPGA or the GPIO to
    control the switch, but not both. When using the   FPGA, ON (OFF) would
    mean that the FPGA outputs a HIGH (LOW) signal continuously.
    """

    FAST_SWITCH_MODE_UNSPECIFIED = 0
    FAST_SWITCH_MODE_ON = 1
    FAST_SWITCH_MODE_OFF = 2
    FAST_SWITCH_MODE_DIRECT = 3
    FAST_SWITCH_MODE_INVERTED = 4


class RfDownConvUpdateRfInput(betterproto.Enum):
    """
    RF Input:    The are 5 debug inputs (P4, P6, P7, P9 & P3) and the main
    input (P1).    SW2 select between the main input and one of the debug
    inputs    SW3 selects between P3 and the rest of the debug inputs    SW4 &
    SW5 selects one of the rest of the debug inputs    SW_EN will make the 4 to
    1 selector to be disconnected          |           DEBUG inputs           |
    |        |          | 1-P4 | 2-P6 | 3-P7 | 4-P9 | 5-P3 | Main | FPGA   |
    DISCONNECT
    ------+------+------+------+------+------+------+--------+------------
    SW2 | HIGH | HIGH | HIGH | HIGH | HIGH | LOW  | HIGH-Z | HIGH      SW3 |
    LOW  | LOW  | LOW  | LOW  | HIGH | LOW  | HIGH-Z | LOW      SW4 | LOW  |
    HIGH | LOW  | HIGH | LOW  | LOW  | HIGH-Z | LOW      SW5 | LOW  | LOW  |
    HIGH | HIGH | LOW  | LOW  | HIGH-Z | LOW    SW_EN | LOW  | LOW  | LOW  |
    LOW  | HIGH | HIGH | HIGH-Z | HIGH    When the output of the 4 way switch
    is not to be connected,    we will disable it (SW_EN). In this case, it
    does not matter    what SW4 & SW5 are, we set it to LOW.
    """

    RF_INPUT_UNSPECIFIED = 0
    RF_INPUT_DEBUG_1 = 1
    RF_INPUT_DEBUG_2 = 2
    RF_INPUT_DEBUG_3 = 3
    RF_INPUT_DEBUG_4 = 4
    RF_INPUT_DEBUG_5 = 5
    RF_INPUT_MAIN = 6
    RF_INPUT_FPGA_CONTROL = 7
    RF_INPUT_DISCONNECT = 8


class RfDownConvUpdateLoInput(betterproto.Enum):
    """
    LO Input:    The are 2 optional LO inputs P8 & P10. SW1 selects between
    them.        | 1-P8  | 2-P10 | FPGA      ----+-------+-------+--------
    SW1 | HIGH  | LOW   | HIGH-Z
    """

    LO_INPUT_UNSPECIFIED = 0
    LO_INPUT_1 = 1
    LO_INPUT_2 = 2
    LO_INPUT_FPGA_CONTROL = 3


class IfDownConvUpdateMode(betterproto.Enum):
    """
    Mode   Two switches are programmed (CTRL6 and CTRL1) as well as three power
    switches (IF_EN1 - right   brach, IF_EN2 - mixer input and DET_EN - power
    detector)                 | CTRL6  | CTRL1  | IF_EN1 | IF_EN2 | DET_EN
    --------------+--------+--------+--------+--------+--------    OFF
    | HIGH   | HIGH   | LOW    | LOW    | LOW    BYPASS       | HIGH   | LOW
    | HIGH   | LOW    | LOW    POWER_DETECT | HIGH   | HIGH   | HIGH   | LOW
    | HIGH    MIXER        | LOW    | LOW    | LOW    | HIGH   | LOW
    """

    MODE_UNSPECIFIED = 0
    MODE_OFF = 1
    MODE_BYPASS = 2
    MODE_POWER_DETECT = 3
    MODE_MIXER = 4


class IfDownConvUpdateCoupling(betterproto.Enum):
    """
    Mode   Two switches are programmed (CTRL4 and CTRL5)                 |
    CTRL4  | CTRL5   --------------+--------+--------    OPEN         | LOW
    | LOW    DC           | HIGH   | LOW    AC           | LOW    | HIGH
    """

    COUPLING_UNSPECIFIED = 0
    COUPLING_OPEN = 1
    COUPLING_DC = 2
    COUPLING_AC = 3


class ClockUpdateMode(betterproto.Enum):
    MODE_UNSPECIFIED = 0
    MODE_BUFFERED = 1
    MODE_EXTERNAL_USE_1G = 2
    MODE_EXTERNAL = 3
    MODE_INTERNAL_USE_1G = 4
    MODE_INTERNAL = 5


class AdminClockUpdateMode(betterproto.Enum):
    MODE_UNSPECIFIED = 0
    MODE_BUFFERED = 1
    MODE_EXTERNAL_USE_1G = 2
    MODE_EXTERNAL = 3
    MODE_INTERNAL_USE_1G = 4
    MODE_INTERNAL = 5


class SynthUpdateReferenceSource(betterproto.Enum):
    REFERENCE_SOURCE_UNSPECIFIED = 0
    REFERENCE_SOURCE_INTERNAL = 1
    REFERENCE_SOURCE_EXTERNAL = 2


class SynthUpdateSynthOutputPower(betterproto.Enum):
    SYNTH_OUTPUT_POWER_UNSPECIFIED = 0
    SYNTH_OUTPUT_POWER_NEG4DB = 1
    SYNTH_OUTPUT_POWER_NEG1DB = 2
    SYNTH_OUTPUT_POWER_POS2DB = 3
    SYNTH_OUTPUT_POWER_POS5DB = 4


class SynthUpdateMainSource(betterproto.Enum):
    MAIN_SOURCE_UNSPECIFIED = 0
    MAIN_SOURCE_EXTERNAL = 1
    MAIN_SOURCE_SYNTHESIZER = 2


class SynthUpdateMainOutput(betterproto.Enum):
    MAIN_OUTPUT_UNSPECIFIED = 0
    MAIN_OUTPUT_MAIN = 1
    MAIN_OUTPUT_OFF = 2
    MAIN_OUTPUT_FPGA_CONTROL = 3


class SynthUpdateSecondaryOutput(betterproto.Enum):
    SECONDARY_OUTPUT_UNSPECIFIED = 0
    SECONDARY_OUTPUT_MAIN = 1
    SECONDARY_OUTPUT_AUXILARY = 2
    SECONDARY_OUTPUT_OFF = 3
    SECONDARY_OUTPUT_FPGA_CONTROL = 4


class SynthRfOutputOutputPort(betterproto.Enum):
    OUTPUT_PORT_UNSPECIFIED = 0
    OUTPUT_PORT_SYNTH = 1
    OUTPUT_PORT_MAIN = 2
    OUTPUT_PORT_SECONDARY = 3


class RfDownConvIfSourceOutputPort(betterproto.Enum):
    OUTPUT_PORT_UNSPECIFIED = 0
    OUTPUT_PORT_I = 1
    OUTPUT_PORT_Q = 2


@dataclass(eq=False, repr=False)
class AdminInfoRequest(betterproto.Message):
    opx_info: "AdminInfoRequestOpxType" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class AdminInfoResponse(betterproto.Message):
    is_success: bool = betterproto.bool_field(1)
    error_message: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class AdminResetRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class AdminResetResponse(betterproto.Message):
    is_success: bool = betterproto.bool_field(1)
    error_message: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ResetRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ResetResponse(betterproto.Message):
    is_success: bool = betterproto.bool_field(1)
    error_message: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class HealthRequest(betterproto.Message):
    """
    Server expects one HealthRequest, other messages are not used server gRPC
    is sync => block until the stream ends
    """

    monitor_interval_seconds: int = betterproto.uint32_field(1)
    """
    Request monitor interval min value:
    monitor_interval=MIN(monitor_interval_seconds,monitor_interval_debug_param)
    """

    stop_stream: bool = betterproto.bool_field(2)
    """Not used, left for future code (Async gRPC)"""


@dataclass(eq=False, repr=False)
class HealthResponse(betterproto.Message):
    explore: "ExploreResponse" = betterproto.message_field(1)
    monitor: "MonitorResponse" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class GetVersionRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetVersionResponse(betterproto.Message):
    version: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ModuleReference(betterproto.Message):
    type: "OctaveModule" = betterproto.enum_field(1)
    index: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class SaveRequest(betterproto.Message):
    id: str = betterproto.string_field(1)
    modules: List["ModuleReference"] = betterproto.message_field(2)
    overwrite: bool = betterproto.bool_field(3)
    timestamp: int = betterproto.int64_field(4)


@dataclass(eq=False, repr=False)
class SaveResponse(betterproto.Message):
    success: bool = betterproto.bool_field(1)
    message: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class RecallRequest(betterproto.Message):
    id: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class RecallResponse(betterproto.Message):
    success: bool = betterproto.bool_field(1)
    error_message: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ListRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class SaveInfo(betterproto.Message):
    request: "SaveRequest" = betterproto.message_field(1)
    content: "UpdateRequest" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ListResponse(betterproto.Message):
    save_infos: List["SaveInfo"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class MonitorRequest(betterproto.Message):
    sense_only: bool = betterproto.bool_field(1)


@dataclass(eq=False, repr=False)
class MonitorResponse(betterproto.Message):
    modules: List["MonitorResponseModuleStatus"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class MonitorResponseModuleStatusError(betterproto.Message):
    type: "MonitorResponseOctaveError" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class MonitorResponseModuleStatus(betterproto.Message):
    module: "ModuleReference" = betterproto.message_field(1)
    temperature: float = betterproto.float_field(2)
    errors: List["MonitorResponseModuleStatusError"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class ControlRequest(betterproto.Message):
    w_data: bytes = betterproto.bytes_field(1)
    r_length: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class ControlResponse(betterproto.Message):
    r_data: bytes = betterproto.bytes_field(1)
    """
    r_data will hold the number of written bytes OR RdataDebug enum in case of
    debug params
    """


@dataclass(eq=False, repr=False)
class ExploreRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ExploreResponse(betterproto.Message):
    modules: List["ExploreResponseModuleId"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ExploreResponseModuleId(betterproto.Message):
    module: "ModuleReference" = betterproto.message_field(1)
    id: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class UpdateRequest(betterproto.Message):
    updates: List["SingleUpdate"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class UpdateResponse(betterproto.Message):
    success: bool = betterproto.bool_field(1)
    error_message: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class SingleUpdate(betterproto.Message):
    rf_up_conv: "RfUpConvUpdate" = betterproto.message_field(1, group="update")
    rf_down_conv: "RfDownConvUpdate" = betterproto.message_field(2, group="update")
    if_down_conv: "IfDownConvUpdate" = betterproto.message_field(3, group="update")
    synth: "SynthUpdate" = betterproto.message_field(4, group="update")
    clock: "ClockUpdate" = betterproto.message_field(5, group="update")
    motherboard: "MotherboardUpdate" = betterproto.message_field(6, group="update")
    admin_clock: "AdminClockUpdate" = betterproto.message_field(7, group="update")


@dataclass(eq=False, repr=False)
class RfUpConvUpdate(betterproto.Message):
    index: int = betterproto.uint32_field(1)
    input_attn: "RfUpConvUpdateIfInputSelection" = betterproto.enum_field(2)
    enabled: Optional[bool] = betterproto.message_field(3, wraps=betterproto.TYPE_BOOL)
    """
    Enabled Both LO amplifier and the mixer amplifier must be            |
    Enabled | Disabled  ----------+---------+----------  10V_LO_EN | HIGH    |
    LOW      8V_EN | HIGH    | LOW
    """

    mixer_output_attn: Optional[int] = betterproto.message_field(
        4, wraps=betterproto.TYPE_UINT32
    )
    """ATTB_[0-5], 6 bits."""

    power_amp_enabled: Optional[bool] = betterproto.message_field(
        5, wraps=betterproto.TYPE_BOOL
    )
    """
    power_amp_enabled   The amplified mixer output can be amplified again (with
    a varible gain), or   simply bypassed. The power amp should be enabled
    accordigly.          | enabled | disabled   -------+--------+----------
    SW3    | LOW     | HIGH   10V_EN | HIGH    | LOW
    """

    power_amp_attn: Optional[int] = betterproto.message_field(
        6, wraps=betterproto.TYPE_UINT32
    )
    """ATTA_[0-5], 6 bits. (relevant only if rf_pwr_sel is HIGH)"""

    fast_switch_mode: "RfUpConvUpdateFastSwitchMode" = betterproto.enum_field(7)


@dataclass(eq=False, repr=False)
class RfDownConvUpdate(betterproto.Message):
    index: int = betterproto.uint32_field(1)
    rf_input: "RfDownConvUpdateRfInput" = betterproto.enum_field(2)
    lo_input: "RfDownConvUpdateLoInput" = betterproto.enum_field(3)
    enabled: Optional[bool] = betterproto.message_field(4, wraps=betterproto.TYPE_BOOL)
    """P_EN enables the 5.0V power."""


@dataclass(eq=False, repr=False)
class IfDownConvUpdate(betterproto.Message):
    index: int = betterproto.uint32_field(1)
    channel1: "IfDownConvUpdateChannel" = betterproto.message_field(2)
    channel2: "IfDownConvUpdateChannel" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class IfDownConvUpdateChannel(betterproto.Message):
    mode: "IfDownConvUpdateMode" = betterproto.enum_field(1)
    coupling: "IfDownConvUpdateCoupling" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class ClockUpdate(betterproto.Message):
    mode: "ClockUpdateMode" = betterproto.enum_field(2)
    """
    There are there general modes: 1. BUFFERED - The clock distributor will use
    the external clock and apply divider at most (no PLL) 2. INTERNAL - Use
    only internal crystals to achieve the desired output clocks 3. EXTERNAL -
    Use PLLs to lock the VCO on an external clock Both 1. and 3. requires the
    frequency of the external clock. In case 2., the clock_frequency field is
    ignored. Modes 2. and 3. has a variant of using an external 1GHz VCXO,
    instead of the LMK internal 3GHz VCO. clock_frequency is in Hz.
    """

    clock_frequency: Optional[float] = betterproto.message_field(
        3, wraps=betterproto.TYPE_DOUBLE
    )
    synthesizers_clock: Optional[float] = betterproto.message_field(
        4, wraps=betterproto.TYPE_DOUBLE
    )
    """
    This is the clock frequency expected by the synthesizers (by default it is
    125MHz, but you may change it). It is important to understand the relation
    between the synthesizers_clock and the above congiuration. The clocks sent
    to the synthesizers must be an interger division of the "main" clock. The
    ratio can not exceed 31. In mode 1., the "main" clock is simply the
    external clock. For example, if you use an OPT's output (1GHz) and the
    desired synthesizer's clock is 125MHz, it is a division by 8 (which is
    perfectly fine). You can use the 500MHz NEL source as well, and the
    division will be 4. In modes 2. and 3., if NOT using the external 1GHz
    VCXO, we first lock an internal 3GHz VCO and this is the "main" clock.
    Again, if we assume 125MHz clock for the synthesizers, the divider is 24.
    This is absolutely fine as well. In modes 2. and 3., if USING the external
    1GHz VCXO, the "main" clock is 1GHz. Note that, for example, you can clock
    the synthesizers at 62.5MHz if using 1GHz VXO, but not if you don't!
    """


@dataclass(eq=False, repr=False)
class AdminClockUpdate(betterproto.Message):
    mode: "AdminClockUpdateMode" = betterproto.enum_field(2)
    """
    There are there general modes: 1. BUFFERED - The clock distributor will use
    the external clock and apply divider at most (no PLL) 2. INTERNAL - Use
    only internal crystals to achieve the desired output clocks 3. EXTERNAL -
    Use PLLs to lock the VCO on an external clock Both 1. and 3. requires the
    frequency of the external clock. In case 2., the clock_frequency field is
    ignored. Modes 2. and 3. has a variant of using an external 1GHz VCXO,
    instead of the LMK internal 3GHz VCO. clock_frequency is in Hz.
    """

    clock_frequency: Optional[float] = betterproto.message_field(
        3, wraps=betterproto.TYPE_DOUBLE
    )
    synthesizers_clock: Optional[float] = betterproto.message_field(
        4, wraps=betterproto.TYPE_DOUBLE
    )
    """
    This is the clock frequency expected by the synthesizers (by default it is
    125MHz, but you may change it). It is important to understand the relation
    between the synthesizers_clock and the above congiuration. The clocks sent
    to the synthesizers must be an interger division of the "main" clock. The
    ratio can not exceed 31. In mode 1., the "main" clock is simply the
    external clock. For example, if you use an OPT's output (1GHz) and the
    desired synthesizer's clock is 125MHz, it is a division by 8 (which is
    perfectly fine). You can use the 500MHz NEL source as well, and the
    division will be 4. In modes 2. and 3., if NOT using the external 1GHz
    VCXO, we first lock an internal 3GHz VCO and this is the "main" clock.
    Again, if we assume 125MHz clock for the synthesizers, the divider is 24.
    This is absolutely fine as well. In modes 2. and 3., if USING the external
    1GHz VCXO, the "main" clock is 1GHz. Note that, for example, you can clock
    the synthesizers at 62.5MHz if using 1GHz VXO, but not if you don't!
    """


@dataclass(eq=False, repr=False)
class MotherboardUpdate(betterproto.Message):
    fan_speed: Optional[float] = betterproto.message_field(
        1, wraps=betterproto.TYPE_DOUBLE
    )


@dataclass(eq=False, repr=False)
class SynthUpdate(betterproto.Message):
    index: int = betterproto.uint32_field(1)
    reference_clock: "SynthUpdateReferenceClock" = betterproto.message_field(2)
    synth_output: "SynthUpdateSynthOutput" = betterproto.message_field(3)
    synth_output_power: "SynthUpdateSynthOutputPower" = betterproto.enum_field(4)
    heater: Optional[int] = betterproto.message_field(5, wraps=betterproto.TYPE_UINT32)
    gain: Optional[int] = betterproto.message_field(6, wraps=betterproto.TYPE_UINT32)
    digital_attn: Optional[int] = betterproto.message_field(
        7, wraps=betterproto.TYPE_UINT32
    )
    """
    Digital attenuator   If the internal synth. generates any tone, the right
    channel must be selected and then it reaches a    programable attenuator.
    The attenuation will be a 6-bits number times 0.5dB.   If specified, the
    following steps are applied:   1. set DIGI_ATT_EN to LOW (*)   2. set
    DIGI_ATT[5:0] to digital_attn   3. set DIGI_ATT_EN to HIGH   (*)
    DIGI_ATT_EN is constantly HIGH in general.
    """

    main_source: "SynthUpdateMainSource" = betterproto.enum_field(8)
    """
    Main source    The main source select between the EXTERNAL LO input and the
    internal synth's output.         | EXTERNAL | SYNTHESIZER | FPGA_CONTROL
    -----+----------+-------------+---------------    SW3  | HIGH     | LOW
    | HIGH_Z
    """

    main_output: "SynthUpdateMainOutput" = betterproto.enum_field(9)
    """
    Main output    The main output (P6) is either either ON or OFF. When it is
    on, it will    output the LO-source after amplification.         | ON   |
    OFF  | FPGA_CONTROL    -----+------+------+---------------    SW6  | LOW  |
    HIGH | HIGH_Z
    """

    secondary_output: "SynthUpdateSecondaryOutput" = betterproto.enum_field(10)
    """
    Secondary output    The secondary output (P4) can output the LO-source
    (like the main output), the    additional external LO source (P5) or none
    (OFF).    output the LO-source after amplification.         | MAIN  | OFF
    | AUXILARY_LO | FPGA_CONTROL
    -----+-------+------+-------------+----------------------    SW4  | HIGH  |
    HIGH | LOW         | HIGH_Z    SW5  | HIGH  | LOW  | LOW         | HIGH_Z
    """

    stabilizer: "SynthUpdateStabilizer" = betterproto.message_field(11)


@dataclass(eq=False, repr=False)
class SynthUpdateReferenceClock(betterproto.Message):
    source: "SynthUpdateReferenceSource" = betterproto.enum_field(1)
    frequency: Optional[float] = betterproto.message_field(
        2, wraps=betterproto.TYPE_DOUBLE
    )
    divider: Optional[int] = betterproto.message_field(3, wraps=betterproto.TYPE_INT32)


@dataclass(eq=False, repr=False)
class SynthUpdateSynthOutput(betterproto.Message):
    frequency: Optional[float] = betterproto.message_field(
        1, wraps=betterproto.TYPE_DOUBLE, group="update"
    )
    disabled: Optional[bool] = betterproto.message_field(
        2, wraps=betterproto.TYPE_BOOL, group="update"
    )


@dataclass(eq=False, repr=False)
class SynthUpdateStabilizer(betterproto.Message):
    k_p: Optional[float] = betterproto.message_field(1, wraps=betterproto.TYPE_FLOAT)
    k_i: Optional[float] = betterproto.message_field(2, wraps=betterproto.TYPE_FLOAT)
    k_d: Optional[float] = betterproto.message_field(3, wraps=betterproto.TYPE_FLOAT)
    a: Optional[float] = betterproto.message_field(4, wraps=betterproto.TYPE_FLOAT)
    b: Optional[float] = betterproto.message_field(5, wraps=betterproto.TYPE_FLOAT)
    heater_max: Optional[int] = betterproto.message_field(
        6, wraps=betterproto.TYPE_UINT32
    )
    set_point: Optional[float] = betterproto.message_field(
        7, wraps=betterproto.TYPE_FLOAT
    )
    enabled: Optional[bool] = betterproto.message_field(8, wraps=betterproto.TYPE_BOOL)


@dataclass(eq=False, repr=False)
class AquireRequest(betterproto.Message):
    modules: List["ModuleReference"] = betterproto.message_field(1)
    use_cache: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class AquireResponse(betterproto.Message):
    state: "UpdateRequest" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class IdentifyRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class IdentifyResponse(betterproto.Message):
    """IdentifyResponse"""

    rf_up_converters: List["RfUpConvIdentity"] = betterproto.message_field(1)
    rf_down_converters: List["RfDownConvIdentity"] = betterproto.message_field(2)
    if_down_converters: List["IfDownConvIdentity"] = betterproto.message_field(3)
    synthesizers: List["SynthIdentity"] = betterproto.message_field(4)
    panel_identity: "PanelIdentity" = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class UpConvRfOutput(betterproto.Message):
    """
    RF UpConv output    The RF up conv. board has only one output. This is the
    modulated signal. The board index is    sufficient to identify it.
    """

    index: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class SynthRfOutput(betterproto.Message):
    """
    Synthesizer output    The synthesizer board has three output:    1. SYNTH
    output:            The output of the synth. is split. One part is routed to
    an SMA connector (this            is the Synth. output, P1). The other part
    will potentially make its way to the             other outputs.
    (*) Note that the internal Synth may be disabled and nothing will come out
    of this            output.    2. MAIN output            One of the
    (amplified) Synth. output OR the (amplified) main LO input (P3) OR zero
    signal may be routed to this output.                         (*) This
    outpot port is intended to be connected to an UpConv. board LO input,
    but not neccessary.                3. SECONDATY output            This port
    can either output a copy of the MAIN output OR the secondary input LO
    (P5).                        (*) Outputing two copies of the same LO is
    needed for readout, or to keep coherence            between two qubits.
    """

    index: int = betterproto.uint32_field(1)
    output_port: "SynthRfOutputOutputPort" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class ExternalRfInput(betterproto.Message):
    lo_input_index: int = betterproto.uint32_field(1, group="input")
    demod_lo_input_index: int = betterproto.uint32_field(2, group="input")
    rf_in_index: int = betterproto.uint32_field(3, group="input")


@dataclass(eq=False, repr=False)
class RfSource(betterproto.Message):
    rf_up_conv_output: "UpConvRfOutput" = betterproto.message_field(1, group="source")
    synth_output: "SynthRfOutput" = betterproto.message_field(2, group="source")
    external_input: "ExternalRfInput" = betterproto.message_field(3, group="source")
    constant_source: "ConstantSource" = betterproto.enum_field(4, group="source")


@dataclass(eq=False, repr=False)
class RfDownConvIfSource(betterproto.Message):
    index: int = betterproto.uint32_field(1)
    output_port: "RfDownConvIfSourceOutputPort" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class IfDownConvIfSource(betterproto.Message):
    index: int = betterproto.uint32_field(1)
    channel_index: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class ExternalIfInput(betterproto.Message):
    i_index: int = betterproto.uint32_field(1, group="input")
    q_index: int = betterproto.uint32_field(2, group="input")
    if_lo_i_index: int = betterproto.uint32_field(3, group="input")
    if_lo_q_index: int = betterproto.uint32_field(4, group="input")


@dataclass(eq=False, repr=False)
class IfSource(betterproto.Message):
    rf_downconv_if_source: "RfDownConvIfSource" = betterproto.message_field(
        1, group="source"
    )
    if_downconv_if_source: "IfDownConvIfSource" = betterproto.message_field(
        2, group="source"
    )
    external_if_input: "ExternalIfInput" = betterproto.message_field(3, group="source")
    constant_source: "ConstantSource" = betterproto.enum_field(4, group="source")


@dataclass(eq=False, repr=False)
class RfUpConvIdentity(betterproto.Message):
    index: int = betterproto.uint32_field(1)
    connectivity: "RfUpConvIdentityConnectivity" = betterproto.message_field(2)
    parameters: "RfUpConvIdentityParameters" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class RfUpConvIdentityConnectivity(betterproto.Message):
    i_input: "IfSource" = betterproto.message_field(1)
    q_input: "IfSource" = betterproto.message_field(2)
    lo_input: "RfSource" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class RfUpConvIdentityParameters(betterproto.Message):
    attn_1_db: float = betterproto.float_field(1)
    attn_2_db: float = betterproto.float_field(2)


@dataclass(eq=False, repr=False)
class RfDownConvIdentity(betterproto.Message):
    index: int = betterproto.uint32_field(1)
    connectivity: "RfDownConvIdentityConnectivity" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class RfDownConvIdentityConnectivity(betterproto.Message):
    debug_rf_input_1: "RfSource" = betterproto.message_field(1)
    debug_rf_input_2: "RfSource" = betterproto.message_field(2)
    debug_rf_input_3: "RfSource" = betterproto.message_field(3)
    debug_rf_input_4: "RfSource" = betterproto.message_field(4)
    debug_rf_input_5: "RfSource" = betterproto.message_field(5)
    rf_main_input: "RfSource" = betterproto.message_field(6)
    lo_input_1: "RfSource" = betterproto.message_field(7)
    lo_input_2: "RfSource" = betterproto.message_field(8)


@dataclass(eq=False, repr=False)
class IfDownConvIdentity(betterproto.Message):
    index: int = betterproto.uint32_field(1)
    connectivity: "IfDownConvIdentityConnectivity" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class IfDownConvIdentityConnectivity(betterproto.Message):
    channel_1_input: "IfSource" = betterproto.message_field(1)
    channel_1_lo_input: "IfSource" = betterproto.message_field(2)
    channel_2_input: "IfSource" = betterproto.message_field(3)
    channel_2_lo_input: "IfSource" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class SynthIdentity(betterproto.Message):
    index: int = betterproto.uint32_field(1)
    connectivity: "SynthIdentityConnectivity" = betterproto.message_field(2)
    parameters: "SynthIdentityParameters" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class SynthIdentityConnectivity(betterproto.Message):
    main_lo_input: "RfSource" = betterproto.message_field(1)
    secondary_lo_input: "RfSource" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class SynthIdentityParameters(betterproto.Message):
    low_frequency_filters: List[
        "SynthIdentityParametersLowFrequencyFilter"
    ] = betterproto.message_field(1)
    medium_frequency_filter: "SynthIdentityParametersParametrizedFilter" = (
        betterproto.message_field(2)
    )
    high_frequency_filter: "SynthIdentityParametersParametrizedFilter" = (
        betterproto.message_field(3)
    )


@dataclass(eq=False, repr=False)
class SynthIdentityParametersLowFrequencyFilter(betterproto.Message):
    index: int = betterproto.uint32_field(1)
    filter_1: str = betterproto.string_field(2)
    filter_2: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class SynthIdentityParametersParametrizedFilter(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class PanelIdentity(betterproto.Message):
    rf_outputs: List["PanelIdentityRfOutput"] = betterproto.message_field(1)
    if_output_i: List["IfSource"] = betterproto.message_field(2)
    if_output_q: List["IfSource"] = betterproto.message_field(3)
    synth_outputs: List["PanelIdentitySynthOutput"] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class PanelIdentityRfOutput(betterproto.Message):
    index: int = betterproto.uint32_field(1)
    source: "RfSource" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class PanelIdentitySynthOutput(betterproto.Message):
    index: int = betterproto.uint32_field(1)
    source: "RfSource" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class MotherboardStatus(betterproto.Message):
    valid_1_g_vco: bool = betterproto.bool_field(1)


class OctaveServiceStub(betterproto.ServiceStub):
    async def get_version(
        self,
        get_version_request: "GetVersionRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetVersionResponse":
        return await self._unary_unary(
            "/quantummachines.octave.api.v1.OctaveService/GetVersion",
            get_version_request,
            GetVersionResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def control(
        self,
        control_request: "ControlRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ControlResponse":
        return await self._unary_unary(
            "/quantummachines.octave.api.v1.OctaveService/Control",
            control_request,
            ControlResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def update(
        self,
        update_request: "UpdateRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "UpdateResponse":
        return await self._unary_unary(
            "/quantummachines.octave.api.v1.OctaveService/Update",
            update_request,
            UpdateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def aquire(
        self,
        aquire_request: "AquireRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AquireResponse":
        return await self._unary_unary(
            "/quantummachines.octave.api.v1.OctaveService/Aquire",
            aquire_request,
            AquireResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def identify(
        self,
        identify_request: "IdentifyRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "IdentifyResponse":
        return await self._unary_unary(
            "/quantummachines.octave.api.v1.OctaveService/Identify",
            identify_request,
            IdentifyResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def monitor(
        self,
        monitor_request: "MonitorRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MonitorResponse":
        return await self._unary_unary(
            "/quantummachines.octave.api.v1.OctaveService/Monitor",
            monitor_request,
            MonitorResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def explore(
        self,
        explore_request: "ExploreRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ExploreResponse":
        return await self._unary_unary(
            "/quantummachines.octave.api.v1.OctaveService/Explore",
            explore_request,
            ExploreResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def save(
        self,
        save_request: "SaveRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "SaveResponse":
        return await self._unary_unary(
            "/quantummachines.octave.api.v1.OctaveService/Save",
            save_request,
            SaveResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def recall(
        self,
        recall_request: "RecallRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "RecallResponse":
        return await self._unary_unary(
            "/quantummachines.octave.api.v1.OctaveService/Recall",
            recall_request,
            RecallResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def list(
        self,
        list_request: "ListRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ListResponse":
        return await self._unary_unary(
            "/quantummachines.octave.api.v1.OctaveService/List",
            list_request,
            ListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def reset(
        self,
        reset_request: "ResetRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResetResponse":
        return await self._unary_unary(
            "/quantummachines.octave.api.v1.OctaveService/Reset",
            reset_request,
            ResetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def health(
        self,
        health_request_iterator: Union[
            AsyncIterable["HealthRequest"], Iterable["HealthRequest"]
        ],
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> AsyncIterator["HealthResponse"]:
        async for response in self._stream_stream(
            "/quantummachines.octave.api.v1.OctaveService/Health",
            health_request_iterator,
            HealthRequest,
            HealthResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    async def admin_reset(
        self,
        admin_reset_request: "AdminResetRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AdminResetResponse":
        return await self._unary_unary(
            "/quantummachines.octave.api.v1.OctaveService/AdminReset",
            admin_reset_request,
            AdminResetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def admin_info(
        self,
        admin_info_request: "AdminInfoRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AdminInfoResponse":
        return await self._unary_unary(
            "/quantummachines.octave.api.v1.OctaveService/AdminInfo",
            admin_info_request,
            AdminInfoResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class OctaveServiceBase(ServiceBase):
    async def get_version(
        self, get_version_request: "GetVersionRequest"
    ) -> "GetVersionResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def control(self, control_request: "ControlRequest") -> "ControlResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def update(self, update_request: "UpdateRequest") -> "UpdateResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def aquire(self, aquire_request: "AquireRequest") -> "AquireResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def identify(self, identify_request: "IdentifyRequest") -> "IdentifyResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def monitor(self, monitor_request: "MonitorRequest") -> "MonitorResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def explore(self, explore_request: "ExploreRequest") -> "ExploreResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def save(self, save_request: "SaveRequest") -> "SaveResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def recall(self, recall_request: "RecallRequest") -> "RecallResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list(self, list_request: "ListRequest") -> "ListResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def reset(self, reset_request: "ResetRequest") -> "ResetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def health(
        self, health_request_iterator: AsyncIterator["HealthRequest"]
    ) -> AsyncIterator["HealthResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def admin_reset(
        self, admin_reset_request: "AdminResetRequest"
    ) -> "AdminResetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def admin_info(
        self, admin_info_request: "AdminInfoRequest"
    ) -> "AdminInfoResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_version(
        self, stream: "grpclib.server.Stream[GetVersionRequest, GetVersionResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_version(request)
        await stream.send_message(response)

    async def __rpc_control(
        self, stream: "grpclib.server.Stream[ControlRequest, ControlResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.control(request)
        await stream.send_message(response)

    async def __rpc_update(
        self, stream: "grpclib.server.Stream[UpdateRequest, UpdateResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.update(request)
        await stream.send_message(response)

    async def __rpc_aquire(
        self, stream: "grpclib.server.Stream[AquireRequest, AquireResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.aquire(request)
        await stream.send_message(response)

    async def __rpc_identify(
        self, stream: "grpclib.server.Stream[IdentifyRequest, IdentifyResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.identify(request)
        await stream.send_message(response)

    async def __rpc_monitor(
        self, stream: "grpclib.server.Stream[MonitorRequest, MonitorResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.monitor(request)
        await stream.send_message(response)

    async def __rpc_explore(
        self, stream: "grpclib.server.Stream[ExploreRequest, ExploreResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.explore(request)
        await stream.send_message(response)

    async def __rpc_save(
        self, stream: "grpclib.server.Stream[SaveRequest, SaveResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.save(request)
        await stream.send_message(response)

    async def __rpc_recall(
        self, stream: "grpclib.server.Stream[RecallRequest, RecallResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.recall(request)
        await stream.send_message(response)

    async def __rpc_list(
        self, stream: "grpclib.server.Stream[ListRequest, ListResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.list(request)
        await stream.send_message(response)

    async def __rpc_reset(
        self, stream: "grpclib.server.Stream[ResetRequest, ResetResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.reset(request)
        await stream.send_message(response)

    async def __rpc_health(
        self, stream: "grpclib.server.Stream[HealthRequest, HealthResponse]"
    ) -> None:
        request = stream.__aiter__()
        await self._call_rpc_handler_server_stream(
            self.health,
            stream,
            request,
        )

    async def __rpc_admin_reset(
        self, stream: "grpclib.server.Stream[AdminResetRequest, AdminResetResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.admin_reset(request)
        await stream.send_message(response)

    async def __rpc_admin_info(
        self, stream: "grpclib.server.Stream[AdminInfoRequest, AdminInfoResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.admin_info(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/quantummachines.octave.api.v1.OctaveService/GetVersion": grpclib.const.Handler(
                self.__rpc_get_version,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetVersionRequest,
                GetVersionResponse,
            ),
            "/quantummachines.octave.api.v1.OctaveService/Control": grpclib.const.Handler(
                self.__rpc_control,
                grpclib.const.Cardinality.UNARY_UNARY,
                ControlRequest,
                ControlResponse,
            ),
            "/quantummachines.octave.api.v1.OctaveService/Update": grpclib.const.Handler(
                self.__rpc_update,
                grpclib.const.Cardinality.UNARY_UNARY,
                UpdateRequest,
                UpdateResponse,
            ),
            "/quantummachines.octave.api.v1.OctaveService/Aquire": grpclib.const.Handler(
                self.__rpc_aquire,
                grpclib.const.Cardinality.UNARY_UNARY,
                AquireRequest,
                AquireResponse,
            ),
            "/quantummachines.octave.api.v1.OctaveService/Identify": grpclib.const.Handler(
                self.__rpc_identify,
                grpclib.const.Cardinality.UNARY_UNARY,
                IdentifyRequest,
                IdentifyResponse,
            ),
            "/quantummachines.octave.api.v1.OctaveService/Monitor": grpclib.const.Handler(
                self.__rpc_monitor,
                grpclib.const.Cardinality.UNARY_UNARY,
                MonitorRequest,
                MonitorResponse,
            ),
            "/quantummachines.octave.api.v1.OctaveService/Explore": grpclib.const.Handler(
                self.__rpc_explore,
                grpclib.const.Cardinality.UNARY_UNARY,
                ExploreRequest,
                ExploreResponse,
            ),
            "/quantummachines.octave.api.v1.OctaveService/Save": grpclib.const.Handler(
                self.__rpc_save,
                grpclib.const.Cardinality.UNARY_UNARY,
                SaveRequest,
                SaveResponse,
            ),
            "/quantummachines.octave.api.v1.OctaveService/Recall": grpclib.const.Handler(
                self.__rpc_recall,
                grpclib.const.Cardinality.UNARY_UNARY,
                RecallRequest,
                RecallResponse,
            ),
            "/quantummachines.octave.api.v1.OctaveService/List": grpclib.const.Handler(
                self.__rpc_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListRequest,
                ListResponse,
            ),
            "/quantummachines.octave.api.v1.OctaveService/Reset": grpclib.const.Handler(
                self.__rpc_reset,
                grpclib.const.Cardinality.UNARY_UNARY,
                ResetRequest,
                ResetResponse,
            ),
            "/quantummachines.octave.api.v1.OctaveService/Health": grpclib.const.Handler(
                self.__rpc_health,
                grpclib.const.Cardinality.STREAM_STREAM,
                HealthRequest,
                HealthResponse,
            ),
            "/quantummachines.octave.api.v1.OctaveService/AdminReset": grpclib.const.Handler(
                self.__rpc_admin_reset,
                grpclib.const.Cardinality.UNARY_UNARY,
                AdminResetRequest,
                AdminResetResponse,
            ),
            "/quantummachines.octave.api.v1.OctaveService/AdminInfo": grpclib.const.Handler(
                self.__rpc_admin_info,
                grpclib.const.Cardinality.UNARY_UNARY,
                AdminInfoRequest,
                AdminInfoResponse,
            ),
        }



class QmOctaveException(Exception):
    pass


class ConflictException(QmOctaveException):
    pass


class InternalShareConflictException(ConflictException):
    pass


class InternalExternalOverrideException(ConflictException):
    pass


class CouplingConflictException(ConflictException):
    pass


class LoopbackInternalShareConflictException(ConflictException):
    pass



from enum import Enum
from octave_sdk.grpc.quantummachines.octave.api.v1 import OctaveModule
from typing import Dict


class RFInputRFSource(Enum):
    """ """

    Loopback_RF_out_1 = 1
    Loopback_RF_out_2 = 2
    Loopback_RF_out_3 = 3
    Loopback_RF_out_4 = 4
    Loopback_RF_out_5 = 5
    RF_in = 6
    Off = 7


synth_slot_index_to_panel_mapping: Dict[int, str] = {
    1: "5",
    2: "6",
    3: "1",
    4: "calibration",
    5: "3",
    6: "2",
}


def slot_index_to_panel_mapping(index: int, module_type: OctaveModule):
    if module_type == OctaveModule.OCTAVE_MODULE_SYNTHESIZER:
        return synth_slot_index_to_panel_mapping[index]

    return index


octave_module_to_module_name_mapping: Dict[OctaveModule, str] = {
    OctaveModule.OCTAVE_MODULE_SYNTHESIZER: "Synthesizer",
    OctaveModule.OCTAVE_MODULE_IF_DOWNCONVERTER: "IF",
    OctaveModule.OCTAVE_MODULE_RF_DOWNCONVERTER: "RF Down-converter",
    OctaveModule.OCTAVE_MODULE_RF_UPCONVERTER: "RF Up-converter",
    OctaveModule.OCTAVE_MODULE_SOM: "System",
    OctaveModule.OCTAVE_MODULE_MOTHERBOARD: "Motherboard",
}



class ConnectivityException(Exception):
    pass


class InvalidIdentityException(ConnectivityException):
    pass


class LoopbackConfigException(ConnectivityException):
    pass


class ExploreResponseException(ConnectivityException):
    pass


class RfSourceTypeException(ConnectivityException):
    pass


class ExternalInputTypeException(ConnectivityException):
    pass


class LoSourceValueException(ConnectivityException):
    pass



from octave_sdk.connectivity.connectivity import (
    RFInputRFSource,
    OctaveLOSource,
    RFInputLOSource,
    OctaveOutput,
)
from octave_sdk.octave import Octave, ClockType, ClockFrequency, IFMode, RFOutputMode

__all__ = [
    "RFInputRFSource",
    "OctaveLOSource",
    "RFInputLOSource",
    "OctaveOutput",
    "Octave",
    "ClockType",
    "ClockFrequency",
    "IFMode",
    "RFOutputMode",
]



from octave_sdk.batch import BatchSingleton
from octave_sdk.connectivity.connectivity import (
    OctaveLOSource,
    SynthOutputDeviceInfo,
    Connectivity,
)
from octave_sdk.grpc.quantummachines.octave.api.v1 import (
    SynthRfOutputOutputPort,
    SynthUpdateMainSource,
    SynthUpdateMainOutput,
    SynthUpdateSecondaryOutput,
    SynthUpdate,
)
from octave_sdk._octave_client import OctaveClient
from octave_sdk.octave_module_com import _get_synth_state
from octave_sdk.conflict_exceptions import (
    ConflictException,
    InternalShareConflictException,
    LoopbackInternalShareConflictException,
    InternalExternalOverrideException,
    CouplingConflictException,
)


def _is_synth_output_using_main_source_external(
    synth_out_port: SynthRfOutputOutputPort, synth_state: SynthUpdate
) -> bool:
    using, main_sw = _is_synth_output_using_main_source(synth_out_port, synth_state)

    return using and main_sw == SynthUpdateMainSource.MAIN_SOURCE_EXTERNAL


def _is_synth_output_using_internal(synth_out_port: SynthRfOutputOutputPort, synth_state: SynthUpdate):
    using, main_sw = _is_synth_output_using_main_source(synth_out_port, synth_state)

    return using and main_sw == SynthUpdateMainSource.MAIN_SOURCE_SYNTHESIZER


def _is_synth_output_using_main_source(
    synth_out_port: SynthRfOutputOutputPort, synth_state: SynthUpdate
) -> (bool, SynthUpdateMainSource):
    other_port_using_main_source: bool = False
    if synth_out_port == SynthRfOutputOutputPort.OUTPUT_PORT_SECONDARY:
        if (
            synth_state.secondary_output != SynthUpdateSecondaryOutput.SECONDARY_OUTPUT_OFF
            and synth_state.secondary_output != SynthUpdateSecondaryOutput.SECONDARY_OUTPUT_AUXILARY
        ):
            other_port_using_main_source = True
    elif synth_out_port == SynthRfOutputOutputPort.OUTPUT_PORT_MAIN:
        if synth_state.main_output != SynthUpdateMainOutput.MAIN_OUTPUT_OFF:
            other_port_using_main_source = True

    return other_port_using_main_source, synth_state.main_source


class Conflict:
    def __init__(self):
        self.is_conflict: bool = False
        self.conflict_up_or_down: SynthOutputDeviceInfo = None

    def is_must_raise_error(self) -> bool:
        return False

    def get_is_conflict(self):
        return self.is_conflict

    def get_exception_type(self):
        return ConflictException


class InternalShareConflict(Conflict):
    def __init__(
        self,
        connectivity: Connectivity,
        lo_source: OctaveLOSource,
        synth_index: int,
        other_port_using_internal,
        synth_out_other: SynthRfOutputOutputPort,
    ):
        super().__init__()

        if lo_source == OctaveLOSource.Internal and other_port_using_internal:
            self.is_conflict = True
            self.conflict_up_or_down = connectivity.get_rf_up_down_by_synth_output(synth_index, synth_out_other)

    def is_must_raise_error(self) -> bool:
        return False

    def get_is_conflict(self):
        return super().get_is_conflict()

    def get_exception_type(self):
        return InternalShareConflictException

    def __str__(self):
        return "share conflict"


class InternalExternalOverrideConflict(Conflict):
    def __init__(
        self,
        connectivity: Connectivity,
        synth_state: SynthUpdate,
        synth_index,
        main_source_next: SynthUpdateMainSource,
        other_port_using_main_source: bool,
        synth_out_other: SynthRfOutputOutputPort,
    ):
        super().__init__()

        if (
            other_port_using_main_source
            and synth_state.main_source != SynthUpdateMainSource.MAIN_SOURCE_UNSPECIFIED
            and main_source_next != SynthUpdateMainSource.MAIN_SOURCE_UNSPECIFIED
            and main_source_next != synth_state.main_source
        ):
            self.is_conflict = True
            self.conflict_up_or_down = connectivity.get_rf_up_down_by_synth_output(synth_index, synth_out_other)

    def is_must_raise_error(self) -> bool:
        return True

    def get_is_conflict(self):
        return super().get_is_conflict()

    def get_exception_type(self):
        return InternalExternalOverrideException

    def __str__(self):
        return "override the mode"


class LoopbackInternalShareConflict(Conflict):
    def __init__(self, connectivity: Connectivity, client: OctaveClient, lo_source, synth_index):
        super().__init__()
        if lo_source == OctaveLOSource.Off:
            return

        # User set ext lo while the synth origin internal is used scenario
        if lo_source in connectivity.loopbacks:
            synth_loopback_source_index = connectivity.get_synth_index_from_output_port(
                connectivity.loopbacks[lo_source]
            )
            synth_state_loopback_source = _get_synth_state(client, synth_loopback_source_index)
            synth_out_other_loopback = None
            if _is_synth_output_using_internal(SynthRfOutputOutputPort.OUTPUT_PORT_MAIN, synth_state_loopback_source):
                synth_out_other_loopback = SynthRfOutputOutputPort.OUTPUT_PORT_MAIN
            if _is_synth_output_using_internal(
                SynthRfOutputOutputPort.OUTPUT_PORT_SECONDARY, synth_state_loopback_source
            ):
                synth_out_other_loopback = SynthRfOutputOutputPort.OUTPUT_PORT_SECONDARY

            if synth_out_other_loopback is not None:
                self.is_conflict = True
                self.conflict_up_or_down = connectivity.get_rf_up_down_by_synth_output(
                    synth_loopback_source_index, synth_out_other_loopback
                )

        # User set internal while the external loopback is used in another synth
        if lo_source == OctaveLOSource.Internal:
            lo_loopback = None
            for lo, synth_panel in connectivity.loopbacks.items():
                if synth_index == connectivity.get_synth_index_from_output_port(synth_panel):
                    lo_loopback = lo
                    break
            if lo_loopback is not None:
                synth_loopback_sink_index = connectivity.synth_by_lo_source()[lo_loopback]
                synth_state_loopback_sink = _get_synth_state(client, synth_loopback_sink_index)

                synth_out_other_loopback = None
                if _is_synth_output_using_main_source_external(
                    SynthRfOutputOutputPort.OUTPUT_PORT_MAIN, synth_state_loopback_sink
                ):
                    synth_out_other_loopback = SynthRfOutputOutputPort.OUTPUT_PORT_MAIN
                if _is_synth_output_using_main_source_external(
                    SynthRfOutputOutputPort.OUTPUT_PORT_SECONDARY, synth_state_loopback_sink
                ):
                    synth_out_other_loopback = SynthRfOutputOutputPort.OUTPUT_PORT_SECONDARY

                if synth_out_other_loopback is not None:
                    self.is_conflict = True
                    self.conflict_up_or_down = connectivity.get_rf_up_down_by_synth_output(
                        synth_loopback_sink_index, synth_out_other_loopback
                    )

    def is_must_raise_error(self) -> bool:
        return False

    def get_is_conflict(self):
        return super().get_is_conflict()

    def get_exception_type(self):
        return LoopbackInternalShareConflictException

    def __str__(self):
        return "loopback share conflict"


class CouplingConflict(Conflict):
    def __init__(self):
        super().__init__()
        # Not implemented at this stage

    def is_must_raise_error(self) -> bool:
        return False

    def get_is_conflict(self):
        return super().get_is_conflict()

    def get_exception_type(self):
        return CouplingConflictException


class SynthConflicts:
    synth_state: SynthUpdate = None

    def __init__(
        self,
        synth_index: int,
        synth_out_to_be_used: SynthRfOutputOutputPort,
        lo_source: OctaveLOSource,
        connectivity: Connectivity,
        client: OctaveClient,
        request_up_or_down: SynthOutputDeviceInfo,
        ignore_shared_errors: bool,
    ):
        super().__init__()
        self.is_conflict = False
        self.request_up_or_down = request_up_or_down
        self.ignore_shared_errors = ignore_shared_errors
        self.lo_source = lo_source

        if BatchSingleton().is_batch_mode:
            if SynthConflicts.synth_state is None:
                SynthConflicts.synth_state = _get_synth_state(client, synth_index)
        else:
            SynthConflicts.synth_state = _get_synth_state(client, synth_index)

        # calc the next synth main source sw state
        main_source_next = SynthUpdateMainSource.MAIN_SOURCE_UNSPECIFIED
        if lo_source == OctaveLOSource.Off:
            pass
        elif lo_source == OctaveLOSource.Internal:
            main_source_next = SynthUpdateMainSource.MAIN_SOURCE_SYNTHESIZER
        elif synth_out_to_be_used == SynthRfOutputOutputPort.OUTPUT_PORT_MAIN:
            main_source_next = SynthUpdateMainSource.MAIN_SOURCE_EXTERNAL

        # check if using internal in the other port
        if synth_out_to_be_used == SynthRfOutputOutputPort.OUTPUT_PORT_MAIN:
            synth_out_other = SynthRfOutputOutputPort.OUTPUT_PORT_SECONDARY
        else:
            synth_out_other = SynthRfOutputOutputPort.OUTPUT_PORT_MAIN
        other_port_using_internal = _is_synth_output_using_internal(synth_out_other, SynthConflicts.synth_state)
        other_port_using_main_source, _ = _is_synth_output_using_main_source(
            synth_out_other, SynthConflicts.synth_state
        )

        # Check all conflicts types
        self._conflicts = [
            InternalShareConflict(connectivity, lo_source, synth_index, other_port_using_internal, synth_out_other),
            InternalExternalOverrideConflict(
                connectivity,
                SynthConflicts.synth_state,
                synth_index,
                main_source_next,
                other_port_using_main_source,
                synth_out_other,
            ),
            LoopbackInternalShareConflict(connectivity, client, lo_source, synth_index),
            CouplingConflict(),
        ]

        for conflict in self._conflicts:
            if conflict.get_is_conflict():
                self.is_conflict = True
                break

    def print_error(self):
        """
        Prints error/warnings to the user, in case conflicts found

        @return: None
        """
        for conflict in self._conflicts:
            if conflict.get_is_conflict():
                print_string = (
                    f"{self.request_up_or_down.up_or_down.value} {self.request_up_or_down.index} set lo "
                    f"source to {self.lo_source.name} will cause {conflict} with "
                    f"{conflict.conflict_up_or_down.up_or_down.value} {conflict.conflict_up_or_down.index}"
                )
                if conflict.is_must_raise_error() or not self.ignore_shared_errors:
                    print_string = "Error, " + print_string
                    if not conflict.is_must_raise_error():
                        print_string += ". Supress this error by using ignore_shared_errors=True"
                    raise conflict.get_exception_type()(print_string)
                else:
                    print_string = "Warning, " + print_string
                    print(print_string)



from typing import List

import copy

from octave_sdk.connectivity.exceptions import (
    ExploreResponseException,
)
from octave_sdk.grpc.quantummachines.octave.api.v1 import (
    RfSource,
    OctaveModule,
    ConstantSource,
    IfSource,
    ModuleReference,
    IdentifyResponse,
)

from octave_sdk._octave_client import ExploreResult
from octave_sdk.connectivity.connectivity_util_private import (
    clear_rfsource_synth_output_oneof,
    clear_rfsource_up_conv_oneof,
    clear_ifsource_down_conv_oneof,
    RfSourceType,
    get_rf_source_type,
    IfSourceType,
    get_if_source_type,
    get_rf_source_from_synth_panel_output,
    identity_object_to_module_ref_type_mapping,
)


# Must be called after self._identity_response is copied from the original identity
class FilteredWorkingModules:
    def __init__(
        self,
        identity_response_original: IdentifyResponse,
        explore_result: ExploreResult,
    ) -> None:
        self.identity_response_filtered: IdentifyResponse = copy.deepcopy(identity_response_original)
        self.missing_modules: list(ModuleReference) = []
        self._filter_by_working_modules(explore_result)

    def _filter_by_working_modules(self, explore_result: ExploreResult):
        if not isinstance(explore_result, ExploreResult):
            raise ExploreResponseException("Wrong explore response object")

        # ------Start with plain module removal-----
        self._filter_by_identity_module_and_explore_module(
            self.identity_response_filtered.rf_up_converters,
            explore_result.modules[OctaveModule.OCTAVE_MODULE_RF_UPCONVERTER],
        )
        self._filter_by_identity_module_and_explore_module(
            self.identity_response_filtered.rf_down_converters,
            explore_result.modules[OctaveModule.OCTAVE_MODULE_RF_DOWNCONVERTER],
        )
        self._filter_by_identity_module_and_explore_module(
            self.identity_response_filtered.if_down_converters,
            explore_result.modules[OctaveModule.OCTAVE_MODULE_IF_DOWNCONVERTER],
        )
        self._filter_by_identity_module_and_explore_module(
            self.identity_response_filtered.synthesizers, explore_result.modules[OctaveModule.OCTAVE_MODULE_SYNTHESIZER]
        )

        # -----Work on input wiring, go from back to front-----
        synths = [uc.index for uc in self.identity_response_filtered.synthesizers]

        # panel outputs - remove all synths
        self._remove_synths_from_panel_output(synths)

        # RF up - remove all synths
        self._remove_synths_from_rf_up(synths)

        # RF down - remove all synths and Upconv
        self._remove_synths_and_rf_up_from_rf_out(synths)

        # IF down - remove all Down conv
        self._remove_rf_down_from_if()

    def _remove_rf_down_from_if(self):
        rf_ins = [uc.index for uc in self.identity_response_filtered.rf_down_converters]
        if_convs = self.identity_response_filtered.if_down_converters
        for if_conv in if_convs[:]:
            if_source_fields: List[IfSource] = [
                if_source for if_source in vars(if_conv.connectivity).values() if isinstance(if_source, IfSource)
            ]

            for if_source in if_source_fields:
                source_type = get_if_source_type(if_source)
                if source_type == IfSourceType.rf_downconv_if_source:
                    if if_source.rf_downconv_if_source.index not in rf_ins:
                        clear_ifsource_down_conv_oneof(if_source)
                        if_source.constant_source = ConstantSource.CONSTANT_SOURCE_OPEN

    def _remove_synths_and_rf_up_from_rf_out(self, synths):
        rf_outs = [uc.index for uc in self.identity_response_filtered.rf_up_converters]
        down_convs = self.identity_response_filtered.rf_down_converters
        for down_conv in down_convs[:]:
            rf_source_fields: List[RfSource] = [
                rf_source for rf_source in vars(down_conv.connectivity).values() if isinstance(rf_source, RfSource)
            ]

            for rf_source in rf_source_fields:
                source_type = get_rf_source_type(rf_source)
                if source_type == RfSourceType.synth_output:
                    if rf_source.synth_output.index not in synths:
                        clear_rfsource_synth_output_oneof(rf_source)
                        rf_source.constant_source = ConstantSource.CONSTANT_SOURCE_OPEN
                elif source_type == RfSourceType.rf_up_conv_output and rf_source.rf_up_conv_output.index not in rf_outs:
                    clear_rfsource_up_conv_oneof(rf_source)
                    rf_source.constant_source = ConstantSource.CONSTANT_SOURCE_OPEN

    def _remove_synths_from_rf_up(self, synths):
        up_convs = self.identity_response_filtered.rf_up_converters
        for up_conv in up_convs[:]:
            source = up_conv.connectivity.lo_input
            source_type = get_rf_source_type(source)
            if source_type == RfSourceType.synth_output:
                if source.synth_output.index not in synths:
                    up_conv.connectivity.lo_input = RfSource(constant_source=ConstantSource.CONSTANT_SOURCE_OPEN)
            elif source_type == RfSourceType.rf_up_conv_output:
                raise ValueError(f"LO Source {source_type} is not supported")

    def _remove_synths_from_panel_output(self, synths):
        synth_outputs = self.identity_response_filtered.panel_identity.synth_outputs
        remove_list = []
        for panel_synth in synth_outputs:
            source = get_rf_source_from_synth_panel_output(panel_synth.source)
            if source.index not in synths:
                remove_list.append(panel_synth)

        for panel_synth in remove_list:
            synth_outputs.remove(panel_synth)

    def _filter_by_identity_module_and_explore_module(self, identity_modules, explore_modules: ExploreResult):
        index_list = [i for i, v in enumerate(explore_modules) if v is not None]
        remove_list = []
        for module in identity_modules[:]:
            if (module.index - 1) not in index_list:
                self.missing_modules.append(
                    ModuleReference(type=identity_object_to_module_ref_type_mapping[type(module)], index=module.index)
                )
                remove_list.append(module)

        for module in remove_list:
            identity_modules.remove(module)



from octave_sdk.grpc.quantummachines.octave.api.v1 import OctaveModule, MonitorResponseOctaveError

from octave_sdk._octave_client import ExploreResult, MonitorResult, MonitorData
from octave_sdk.connectivity.connectivity import ModulesSlotsFromIdentity
from octave_sdk.connectivity.connectivity_util import octave_module_to_module_name_mapping, slot_index_to_panel_mapping
from octave_sdk.health_client import HealthClient
import logging

logger = logging.getLogger("qm")


class HealthMonitor:
    # Health Monitor responsible to monitor the Octave HW and update the system connectivity accordingly
    # Use case:
    # create the HealthMonitor, monitor thread will start automatically
    # Use run_once get monitor result and reset the connectivity in the called thread
    # call stop() to request the monitor thread to exit
    #
    # On EVERY change in modules status, the complete print of the monitor result will be showed including errors
    # In case of health return without errors, Health check passed will be printed
    # In case of monitor connection to Octave drop unexpectedly, new connection will be made and error will be printed
    def __init__(
        self, client, name: str, reset_connectivity, slots_list: ModulesSlotsFromIdentity, interval_seconds: int = 5
    ):
        """

        @param client: The Octave client for HW communication
        @param name: The name of the device
        @param reset_connectivity: The function to call in case of reset scenario detected
        @param slots_list: list of slots indexes allowed to be used in this octave
        @param interval_seconds: Health check polling interval, set to 0 for manual check using run_once()
        """
        self._interval_seconds = interval_seconds
        self._client = client
        self._octave_name = name
        self._reset_callback = reset_connectivity
        self._slot_list: ModulesSlotsFromIdentity = slots_list

        self._health_monitor = HealthClient(5, self._client, self._health_update)
        self._health_monitor.run_once()
        self._health_monitor.start()

    def _health_update(self, explore_result: ExploreResult, monitor_result: MonitorResult):
        # int the system with the current modules
        self._reset_callback(explore_result)

        errors_found = False
        max_temp = -300
        for module_name, module_list in monitor_result.modules.items():
            for module_index, monitor_data in enumerate(module_list):
                if monitor_data is not None:
                    if monitor_data.temp > max_temp:
                        max_temp = monitor_data.temp
                    if monitor_data.errors and self._is_module_in_identity(module_name, module_index + 1):
                        errors_found = True
                        self._translate_error_to_string(module_name, module_index + 1, monitor_data)

        if not errors_found:
            logger.info(f'Octave "{self._octave_name}" Health check passed, current temperature {int(max_temp)}')

    def _translate_error_to_string(self, module_name: OctaveModule, module_index: int, monitor_data: MonitorData):
        list_of_errors = "".join(' "' + MonitorResponseOctaveError(err.type).name + '"' for err in monitor_data.errors)

        print_string = (
            f'Octave "{self._octave_name}" {octave_module_to_module_name_mapping[module_name]} index {slot_index_to_panel_mapping(module_index, module_name)} error:'
            + list_of_errors
            + f". temp={monitor_data.temp}"
        )
        logger.warning(print_string)

    # Slot should start from 1
    def _is_module_in_identity(self, name: OctaveModule, slot_index: int):
        slot_list = []

        if name == OctaveModule.OCTAVE_MODULE_RF_UPCONVERTER:
            slot_list = self._slot_list.rf_out_list
        elif name == OctaveModule.OCTAVE_MODULE_RF_DOWNCONVERTER:
            slot_list = self._slot_list.rf_in_list
        elif name == OctaveModule.OCTAVE_MODULE_IF_DOWNCONVERTER:
            slot_list = self._slot_list.if_list
        elif name == OctaveModule.OCTAVE_MODULE_SYNTHESIZER:
            slot_list = self._slot_list.synth_list
        elif name == OctaveModule.OCTAVE_MODULE_MOTHERBOARD or name == OctaveModule.OCTAVE_MODULE_SOM:
            slot_list = [1]

        return slot_index in slot_list

    def run_once(self):
        self._health_monitor.run_once()
